;________________________________________________________________________
;
;                          P I C E D I A G . A S M
;________________________________________________________________________
;
; Sample PIC16F84 code by Byon Garrabrant  N6BG  byon@mail.com  Feb 1999
;________________________________________________________________________

	LIST	P=PIC16F84
	include	"P16F84.INC"
	__CONFIG _HS_OSC & _CP_OFF & _PWRTE_ON & _WDT_OFF

;________________________________________________________________________

	cblock	H'0C'			; system variables
		delay1
		delay2
		serialDelayCount
		serialCount
		serialData
		letter
	endc
;________________________________________________________________________

#define	VALID_OUT	PORTB, 0
#define	MODE1_OUT	PORTB, 1
#define	MODE0_OUT	PORTB, 2
#define	TONE_IN		PORTB, 4
#define	OPTION_IN	PORTB, 4
#define	TONE_OUT	PORTB, 5

#define	PTT_OUT		PORTA, 2
#define	PTT_IN		PORTA, 3
#define	MODE_IN		PORTA, 4
;________________________________________________________________________

	ORG	0			; start code
	goto	main

;________________________________________________________________________

#define	SERIAL_IN_PIN		PORTA, 0
#define	SERIAL_OUT_PIN		PORTA, 1
#define	SERIAL_CLOCK_SPEED	D'10000000'
#define	SERIAL_BAUD_RATE	D'4800'

#define	_SBR			SERIAL_BAUD_RATE
#define	_SCS			SERIAL_CLOCK_SPEED
#define	SERIAL_DELAY_TIME	((((_SCS/4+_SBR/2)/_SBR)-D'10'+1)/3)
#define	SERIAL_RECEIVED		serialData, 0
;________________________________________________________________________

serialInit
	bsf	STATUS, RP0		; Select Bank 1
	bsf	SERIAL_IN_PIN		; Setup input
	bcf	SERIAL_OUT_PIN		; Setup output
	bcf	STATUS, RP0		; Select Bank 0
	return
;________________________________________________________________________

serialTransmit
	movwf	serialData		; save the outgoing byte
	movlw	D'10'
	movwf	serialCount		; set the bit count
	clrc				; prep for start bit
serialTransmitLoop
	skpnc				; if sending 1
	 bcf	SERIAL_OUT_PIN		; send low
	skpc				; if sending 0
	 bsf	SERIAL_OUT_PIN		; send high
	movlw	SERIAL_DELAY_TIME
	movwf	serialDelayCount	; Count down from DELAY_TIME
	decfsz	serialDelayCount, F	; Countdown
	 goto	$-1			; keep counting
	setc				; prep for stop bit
	rrf	serialData, F		; roll serialIn bits	
	decfsz	serialCount, F		; if not complete
	 goto	serialTransmitLoop	; loop back for more
	return				; all done
;________________________________________________________________________

serialReceive
	bcf	SERIAL_RECEIVED
	btfss	SERIAL_IN_PIN		; Check for Start bit
	 retlw	0			; not receiving now
	movlw	D'9'			
	movwf	serialCount		; gonna read 9 bits
	movlw	SERIAL_DELAY_TIME/3
	movwf	serialDelayCount	; Count down from DELAY_TIME
	decfsz	serialDelayCount, F	; Countdown
	 goto	$-1			; keep counting
serialReceiveLoop
	clrc				; assume clear carry
	btfss	SERIAL_IN_PIN		; if receiving 0
	 setc				; set carry
	rrf	serialData, F		; roll serialData bits	
	movlw	SERIAL_DELAY_TIME
	movwf	serialDelayCount	; Count down from DELAY_TIME
	decfsz	serialDelayCount, F	; Countdown
	 goto	$-1			; keep counting
	nop				; keep timing w/ transmit
	nop				; keep timing w/ transmit
	decfsz	serialCount, F		; if not complete
	 goto	serialReceiveLoop	; loop back for more
	movfw	serialData		; return the data
	bsf	SERIAL_RECEIVED
	return
;________________________________________________________________________

delay1200				; delay for about 833 us (2083 ins)
					; actuall 2066 w/o call/return
	movlw	D'177'
	movwf	delay1
	movlw	D'3'
	movwf	delay2
delayLoop	
	decfsz	delay1, F		; loop 256/177 times
	 goto	delayLoop
	decfsz	delay2, F		; loop 3 times
	 goto	delayLoop
	return 
;________________________________________________________________________

flash
	bsf	VALID_OUT		; turn VALID OUT ON
	call	flashDelay		; delay
	bcf	VALID_OUT		; turn VALID OUT OFF
	call	flashDelay		; delay
	return

flashDelay
	nop
	decfsz	delay1, F		; loop 256 times
	 goto	flashDelay
	decfsz	delay2, F		; loop 256 times
	 goto	flashDelay
	return 
;________________________________________________________________________

main
	call	serialInit		; Init Serial
	bsf	STATUS, RP0		; Select Bank 1
	bcf	PTT_OUT			; make PTT an output
	bcf	TONE_OUT		; make data an output
	bcf	MODE0_OUT		; make mode0 an output
	bcf	MODE1_OUT		; make mode1 an output
	bcf	VALID_OUT		; make valid an output
	bcf	STATUS, RP0		; SelectBank 0

	bsf	MODE1_OUT		; disable audio tone
	bcf	MODE0_OUT		; disable audio tone
	bcf	PTT_OUT			; clear PTT_OUT

	call	flash			; flash valid 3 times
	call	flash
	call	flash

mainLoop
	btfsc	MODE_IN			; if MODE IN ON
	 goto	mode_3_4
mode_1_2
	btfss	OPTION_IN		; if OPTION IN OFF
	 goto	mode1
	goto	mode2
mode_3_4
	btfss	OPTION_IN		; if OPTION IN OFF
	 goto	mode3
	goto	mode4

mode1					; mode 1, radio off, valid off
	bsf	VALID_OUT		; turn VALID OUT ON
	bsf	MODE1_OUT		; disable audio tone
	bcf	MODE0_OUT		; disable audio tone
	bcf	PTT_OUT			; disable PTT_OUT

	call	serialReceive		; get serialinput
	btfss	SERIAL_RECEIVED
	 goto	mainLoop		; no input
	addlw	1			; add 1 to received char
	call	serialTransmit		; and transmit it

	goto	mainLoop		; do it again

mode2					; mode 2, both tones at 1200 baud
	bcf	VALID_OUT		; turn VALID OUT OFF
	movlw	B'00100000'		; toggle tone_out
	xorwf	PORTB, F
	call	delay1200		; delay for about 1/1200 sec
	goto	toneOn
mode3					; mode 3
	bsf	PORTB, 5		; send 1200 Hz
	movlw	B'00000001'		; toggle valid LED
	xorwf	PORTB, F
	call	flashDelay		; delay
	incf	letter, F		; bump the letter forward
	movfw	letter
	andlw	H'0F'			; keep between A and P
	addlw	'A'
	call	serialTransmit		; send it
	goto	toneOn
mode4 
	movlw	B'00000001'		; toggle valid LED
	xorwf	PORTB, F
	call	flashDelay		; delay Longer
	call	flashDelay		; delay Longer
	call	flashDelay		; delay Longer
	call	flashDelay		; delay Longer
	bcf	PORTB, 5		; send 2200 Hz
	incf	letter, F		; bump the letter forward
	movfw	letter
	andlw	H'0F'			; keep between a and p
	addlw	'a'
	call	serialTransmit		; send it

toneOn
	bcf	MODE1_OUT		; enable audio tone
	bsf	MODE0_OUT		; enable audio tone
	bsf	PTT_OUT			; enable PTT_OUT
	goto	mainLoop		; do it again

	END
;________________________________________________________________________


